import { Box } from '../../classes/box.js';
import { Scene } from '../../classes/scene.js';
import { PostContent } from '../../classes/post-content.js';
import { Expert } from '../../classes/expert.js';
import { ForumNode } from '../../classes/forum-node.js';
import { Network } from '../../classes/network.js';
import { delay, randomID } from '../../util.js';

const rootElement = document.getElementById('scene');
const rootBox = new Box('rootBox', rootElement).flex();

window.scene = new Scene('Forum Network test', rootBox).withSequenceDiagram();

window.author1 = await new Expert('author1', window.scene).initialize();
window.author2 = await new Expert('author2', window.scene).initialize();

window.forumNetwork = new Network();

window.forumNode1 = await new ForumNode('node1', window.scene).initialize(
  window.forumNetwork,
);
window.forumNode2 = await new ForumNode('node2', window.scene).initialize(
  window.forumNetwork,
);
window.forumNode3 = await new ForumNode('node3', window.scene).initialize(
  window.forumNetwork,
);

const processInterval = setInterval(async () => {
  await window.forumNode1.processNextMessage();
  await window.forumNode2.processNextMessage();
  await window.forumNode3.processNextMessage();
}, 100);

// const blockchain = new Blockchain();

window.post1 = new PostContent({ message: 'hi' });
window.post1.id = randomID();
window.post2 = new PostContent({ message: 'hello' }).addCitation(
  window.post1.id,
  1.0,
);

await delay(1000);
await window.author1.submitPostViaNetwork(
  window.forumNode1,
  window.post1,
  50,
);
await delay(1000);
await window.author2.submitPostViaNetwork(
  window.forumNode2,
  window.post2,
  100,
);

await delay(1000);
clearInterval(processInterval);
