import { Box } from '../../../classes/box.js';
import { Scene } from '../../../classes/scene.js';
import { Expert } from '../../../classes/expert.js';
import { PostContent } from '../../../classes/post-content.js';
import { delay } from '../../../util.js';
import params from '../../../params.js';
import { DAO } from '../../../classes/dao.js';

export class ForumTest {
  constructor(options) {
    this.dao = null;
    this.experts = null;
    this.posts = null;
    this.options = {
      defaultDelayMs: 1,
      poolDurationMs: 50,
      ...options,
    };
  }

  async newExpert() {
    const index = this.experts.length;
    const name = `Expert${index + 1}`;
    const expert = await new Expert(this.dao, name).initialize();
    this.experts.push(expert);
    // expert.addValue('rep', () => this.dao.reputation.valueOwnedBy(expert.reputationPublicKey));
    return expert;
  }

  async addPost(author, fee, citations = []) {
    const postIndex = this.posts.length;
    const title = `posts[${postIndex}]`;
    await window.scene.startSection();

    const postContent = new PostContent({}).setTitle(title);
    for (const { postId, weight } of citations) {
      postContent.addCitation(postId, weight);
    }

    const { pool, postId } = await author.submitPostWithFee(
      postContent,
      {
        fee,
        duration: this.options.poolDurationMs,
        tokenLossRatio: 1,
      },
    );
    this.posts.push(postId);
    await delay(this.options.poolDurationMs);
    await pool.evaluateWinningConditions();
    await window.scene.endSection();
    await delay(this.options.defaultDelayMs);
    return postId;
  }

  async setup() {
    const rootElement = document.getElementById('scene');
    const rootBox = new Box('rootBox', rootElement).flex();

    const scene = (window.scene = new Scene('Forum test', rootBox));
    scene.withSequenceDiagram();
    scene.withFlowchart();
    scene.withTable();

    scene.addDisplayValue('c3. stakeForAuthor').set(params.stakeForAuthor);
    scene.addDisplayValue('q2. revaluationLimit').set(params.revaluationLimit);
    scene
      .addDisplayValue('q3. referenceChainLimit')
      .set(params.referenceChainLimit);
    scene.addDisplayValue('q4. leachingValue').set(params.leachingValue);
    scene.addDisplayValue('&nbsp;');

    this.dao = (window.dao = new DAO('DAO'));
    this.forum = this.dao.forum;
    this.experts = (window.experts = []);
    this.posts = (window.posts = []);

    await this.newExpert();
    // await newExpert();
    // await newExpert();

    this.dao.addValue('total value', () => this.dao.reputation.getTotal());
    // this.dao.addValue('total reputation', () => this.dao.forum.getTotalValue());
    this.dao.computeValues();
  }
}
